using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using MeshVR;
using MVR.FileManagementSecure;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

// VAMStory v1.04
//
// Dialog / narration system for VAM

namespace VAMStoryPlugin
{
	public partial class VAMStory : MVRScript
    {
	    public class StoryDialog
	    {
		    private VAMStory _VAMS;

		    public int GroupID;
		    public int DialogID;

		    public JSONStorableString DialogText;
		    public JSONStorableFloat DialogDelay;
		    public EventTrigger triggerOnShowDialog;
		    public EventTrigger triggerOnClickDialog;
		    public JSONStorableStringChooser DialogAvatarPosition;
		    public JSONStorableUrl DialogAvatarUrl;

			protected UIDynamicTextField dialogTitle;
			protected UIDynamicButton triggerOnShowBtn;
			protected UIDynamicButton triggerOnClickBtn;
			protected UIDynamicButton dialogAvatarBtn;
			protected UIDynamicButton dialogRemoveAvatarBtn;
			
			protected List<string> avatarPositionChoices = new List<string> {"Left", "Right"};
			
			protected InputField DialogTextInputField;
		    
		    protected List<UIDynamic> dialogsComponentsUI = new List<UIDynamic>();
		    
		    
		    
		    protected JSONClass dialogsDatas { get; set; }
		    
		    public StoryDialog(VAMStory rootScript, int groupId, int dialogId)
		    {
			    _VAMS = rootScript;
			    dialogsDatas = _VAMS.dialogsDatas;

			    GroupID = groupId;
			    DialogID = dialogId;
				    
			    CreateDialogComponents();
		    }

		    protected void CreateDialogComponents()
		    {
			    // Temporary vars
			    HSVColor hsvcDefault = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
			    UIDynamicTextField tmpTextField;
			    UIDynamicPopup tmpPopup;
			    UIDynamicSlider tmpSlider;
			    UIDynamicColorPicker tmpColor;
			    UIDynamicButton tmpButton;
			    UIDynamicToggle tmpToggle;
			    UIDynamic tmpSpacer;
			    
			    // Creating Dialog components
				// ******* DIALOG TITLE (ID) ***********
				dialogTitle = _VAMS.createStaticDescriptionText("DialogTitle_" + GroupID + "_" + DialogID,"<color=#000><size=35><b>GROUP "+GroupID+" : Dialog "+DialogID+"</b></size></color>",false,55,TextAnchor.MiddleLeft);
				dialogsComponentsUI.Add((UIDynamic) dialogTitle);
				
				// ******* DIALOG TEXT ***********
				string newDefaultText = "Type some text...";
				DialogText = new JSONStorableString(SAVE_PREFIX + GroupID + "_" + DialogID + "-Text", "_default_");
				tmpTextField = _VAMS.CreateTextField(DialogText);
				_VAMS.setupTextField(tmpTextField, 550f, false, false);
				InputField tmpInp = tmpTextField.UItext.gameObject.AddComponent<InputField>();
				tmpInp.textComponent = tmpTextField.UItext;
				tmpInp.lineType = InputField.LineType.MultiLineNewline;
				dialogsComponentsUI.Add((UIDynamic) tmpTextField);
				DialogText.valNoCallback = newDefaultText;
				tmpInp.text = newDefaultText;				
				tmpInp.onValueChanged.AddListener( delegate { _VAMS.OnDialogTextChanged(DialogText,tmpInp); } );
				DialogTextInputField = tmpInp;
				
				// ******* DIALOG START DELAY ***********
				DialogDelay = new JSONStorableFloat(SAVE_PREFIX + GroupID + "_" + DialogID + "-Delay", 0.2f, 0f, 5f, false);
				tmpSlider = _VAMS.CreateSlider(DialogDelay);
				tmpSlider.valueFormat = "F1";
				tmpSlider.label = "Dialog start delay";
				dialogsComponentsUI.Add((UIDynamic) tmpSlider);

				// ******* TRIGGERS ***********
				triggerOnShowDialog = new EventTrigger(_VAMS, SAVE_PREFIX + GroupID + "_" + DialogID + "-TriggerOnShow");
				triggerOnClickDialog = new EventTrigger(_VAMS, SAVE_PREFIX + GroupID + "_" + DialogID + "-TriggerOnClick");

				triggerOnShowBtn = _VAMS.CreateButton("On show trigger", false);
				dialogsComponentsUI.Add((UIDynamic)triggerOnShowBtn);
				triggerOnShowBtn.button.onClick.AddListener(() => { triggerOnShowDialog.OpenPanelActionStart(); } );
				_VAMS.setupButtonWithLayout(triggerOnShowBtn, 250f);
				
				triggerOnClickBtn = _VAMS.CreateButton("On click trigger", false);
				dialogsComponentsUI.Add((UIDynamic)triggerOnClickBtn);
				_VAMS.ignoreLayoutUI.Add((UIDynamic)triggerOnClickBtn);
				triggerOnClickBtn.button.onClick.AddListener(() => { triggerOnClickDialog.OpenPanelActionStart(); } );
				_VAMS.setupButtonWithoutLayout(triggerOnClickBtn, 250f, new Vector2(270f, -805f));

				// ******* AVATAR POSITION ***********
				DialogAvatarPosition = new JSONStorableStringChooser(SAVE_PREFIX + GroupID + "_" + DialogID + "-AvatarPosition", avatarPositionChoices, "Right", "Avatar position");
				tmpPopup = _VAMS.CreateScrollablePopup(DialogAvatarPosition);
				tmpPopup.labelWidth = 220f;
				dialogsComponentsUI.Add((UIDynamic) tmpPopup);
				
				// ******* AVATAR SELECTION ***********
				dialogAvatarBtn = _VAMS.CreateButton("Select avatar", false);
				_VAMS.setButtonColor(dialogAvatarBtn, new Color(0.7f,1f,0.7f,1f) );
				dialogsComponentsUI.Add((UIDynamic)dialogAvatarBtn);
				dialogAvatarBtn.button.onClick.AddListener(() => { _VAMS.OnSelectDialogAvatar(GroupID, DialogID); } );
				_VAMS.setupButtonWithLayout(dialogAvatarBtn, 250f);
				
				dialogRemoveAvatarBtn = _VAMS.CreateButton("Remove avatar", false);
				_VAMS.setButtonColor(dialogRemoveAvatarBtn, new Color(1f,0.7f,0.7f,1f) );
				dialogsComponentsUI.Add((UIDynamic)dialogRemoveAvatarBtn);
				_VAMS.ignoreLayoutUI.Add((UIDynamic)dialogRemoveAvatarBtn);
				dialogRemoveAvatarBtn.button.onClick.AddListener(() => { DialogAvatarUrl.val = ""; } );
				_VAMS.setupButtonWithoutLayout(dialogRemoveAvatarBtn, 250f, new Vector2(270f, -985f));
				
				DialogAvatarUrl = new JSONStorableUrl(SAVE_PREFIX + GroupID + "_" + DialogID + "-AvatarUrl", "");
				DialogAvatarUrl.setCallbackFunction += (val) => { _VAMS.OnLoadAvatar(val); };
				tmpTextField = _VAMS.CreateTextField(DialogAvatarUrl);
				_VAMS.setupTextField(tmpTextField, 80f);
				dialogsComponentsUI.Add((UIDynamic)tmpTextField);

				// Registering everything
				RegisterStorables();
				
				// Restoring datas
				if (dialogsDatas != null)
				{
					DialogText.RestoreFromJSON( dialogsDatas, true, true, false);
					DialogDelay.RestoreFromJSON( dialogsDatas, true, true, false);
					DialogAvatarPosition.RestoreFromJSON( dialogsDatas, true, true, false);
					DialogAvatarUrl.RestoreFromJSON( dialogsDatas, true, true, false);
					triggerOnShowDialog.RestoreFromJSON( dialogsDatas, _VAMS.subScenePrefix, _VAMS.mergeRestore, true );
					triggerOnClickDialog.RestoreFromJSON( dialogsDatas, _VAMS.subScenePrefix, _VAMS.mergeRestore, true );
				}
				
				// Updating texts inputs
				tmpInp.text = DialogText.val;
		    }

		    public void UpdateDialogText(string val)
		    {
			    DialogTextInputField.text = val;
		    }
		    
		    public void ToggleDialogUI( bool enabled )
		    {
			    foreach( UIDynamic uidElem in dialogsComponentsUI )
			    {
				    _VAMS.ToggleUIDynamic(enabled, uidElem);
			    }
		    }

		    public void DoClickTrigger()
		    {
			    triggerOnClickDialog.Trigger();
		    }
		    
		    public void DoShowTrigger()
		    {
			    triggerOnShowDialog.Trigger();
		    }

		    public void UpdateID(int newGroupId, int newDialogId)
		    {
			    GroupID = newGroupId;
			    DialogID = newDialogId;

			    // Updating dialog components (param names for the save)
			    UpdateComponentsIDs();
		    }

		    protected void UpdateComponentsIDs()
		    {
			    // Deregister
			    DeregisterStorables();
			    
			    // Updated
			    DialogText.name = SAVE_PREFIX + GroupID + "_" + DialogID + "-Text";
			    DialogDelay.name = SAVE_PREFIX + GroupID + "_" + DialogID + "-Delay";
			    DialogAvatarPosition.name = SAVE_PREFIX + GroupID + "_" + DialogID + "-AvatarPosition";
			    triggerOnShowDialog.Name = SAVE_PREFIX + GroupID + "_" + DialogID + "-TriggerOnShow";
			    triggerOnClickDialog.Name = SAVE_PREFIX + GroupID + "_" + DialogID + "-TriggerOnClick";
			    DialogAvatarUrl.name = SAVE_PREFIX + GroupID + "_" + DialogID + "-AvatarUrl";
			    dialogTitle.text = "<color=#000><size=35><b>GROUP " + GroupID + " : Dialog " + DialogID + "</b></size></color>";
			    
			    // Register
			    RegisterStorables();
		    }
		    
			public void Delete() {
				// Clearing the UI
				Destroy(DialogTextInputField);
				_VAMS.RemoveTextField(dialogTitle);
				_VAMS.RemoveTextField(DialogText);
		    	_VAMS.RemoveSlider(DialogDelay);
				_VAMS.RemovePopup(DialogAvatarPosition);
		    	_VAMS.RemoveTextField((JSONStorableString)DialogAvatarUrl);
		    	_VAMS.RemoveButton(triggerOnShowBtn);
		    	_VAMS.RemoveButton(triggerOnClickBtn);
		    	_VAMS.RemoveButton(dialogAvatarBtn);
		    	_VAMS.RemoveButton(dialogRemoveAvatarBtn);

				// Unregistering vars
				DeregisterStorables();
				
				// Removing triggers
				triggerOnShowDialog.Remove();
				triggerOnClickDialog.Remove();
				
				// Clearing data from the loaded data of the scene (to prevent re-restore after deleting)
				// This only happens when you load a scene and have data loaded from the json
				if (dialogsDatas != null)
				{
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "_" + DialogID + "-Text");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "_" + DialogID + "-Delay");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "_" + DialogID + "-AvatarPosition");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "_" + DialogID + "-TriggerOnShow");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "_" + DialogID + "-TriggerOnClick");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "_" + DialogID + "-AvatarUrl");
				}
			}
			
			protected void DeregisterStorables()
			{
				_VAMS.DeregisterString(DialogText);
				_VAMS.DeregisterFloat(DialogDelay);
				_VAMS.DeregisterUrl(DialogAvatarUrl);
				_VAMS.DeregisterStringChooser(DialogAvatarPosition);
			}
			
			protected void RegisterStorables()
			{
				_VAMS.RegisterString(DialogText);
				_VAMS.RegisterFloat(DialogDelay);
				_VAMS.RegisterUrl(DialogAvatarUrl);
				_VAMS.RegisterStringChooser(DialogAvatarPosition);
			}
	    }
    }
}