using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using MeshVR;
using MVR.FileManagementSecure;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

// VAMStory v1.04
//
// Dialog / narration system for VAM

namespace VAMStoryPlugin
{
	public partial class VAMStory : MVRScript
    {
	    public class StoryGroup
	    {
		    private VAMStory _VAMS;

		    public int GroupID;

			public JSONStorableVector3 GroupPosition;
			public JSONStorableVector3 GroupRotation;
			public EventTrigger GroupTrigger;
		    public JSONStorableBool GroupAnswer01Enabled;
		    public JSONStorableString GroupAnswer01Txt;
		    public EventTrigger GroupAnswer01Trigger;
		    public JSONStorableBool GroupAnswer02Enabled;
		    public JSONStorableString GroupAnswer02Txt;
		    public EventTrigger GroupAnswer02Trigger;
		    public JSONStorableBool GroupAnswer03Enabled;
		    public JSONStorableString GroupAnswer03Txt;
		    public EventTrigger GroupAnswer03Trigger;
		    public JSONStorableStringChooser GroupStoryActions;
		    public JSONStorableBool GroupHideMainDialog;
		    
		    protected List<UIDynamic> groupComponentsUI = new List<UIDynamic>();
		    protected List<StoryDialog> DialogList = new List<StoryDialog>();    
		    protected JSONClass dialogsDatas { get; set; }

		    protected UIDynamicButton positionSaveButton;
		    protected UIDynamicButton triggerGroupButton;
		    protected UIDynamicButton triggerAnswer01Button;
		    protected UIDynamicButton triggerAnswer02Button;
		    protected UIDynamicButton triggerAnswer03Button;
		    
		    protected InputField GroupAnswer01TxtInputField;
		    protected InputField GroupAnswer02TxtInputField;
		    protected InputField GroupAnswer03TxtInputField;

			protected Color unsavedPositionColor = new Color(0.5f,0.5f,0.5f,1f);
			protected Color savedPositionColor = new Color(0.1f,1f,0.1f,1f);

		    public StoryGroup(VAMStory rootScript, int groupId)
		    {
			    _VAMS = rootScript;
			    dialogsDatas = _VAMS.dialogsDatas;
			    
			    GroupID = groupId;

			    CreateGroupComponents();
		    }

		    protected void CreateGroupComponents()
		    {
			    // Temporary vars
			    HSVColor hsvcDefault = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
			    UIDynamicTextField tmpTextField;
			    UIDynamicPopup tmpPopup;
			    UIDynamicSlider tmpSlider;
			    UIDynamicColorPicker tmpColor;
			    UIDynamicToggle tmpToggle;
			    UIDynamic tmpSpacer;
			    string newDefaultAnswer = "Answer text...";
			    
				// ******* GROUP POSITION ***********
				GroupPosition = new JSONStorableVector3(SAVE_PREFIX + GroupID + "-Position", new Vector3(0f,0f,0f), new Vector3(0,0,0), new Vector3(0,0,0), false);
				
				GroupRotation = new JSONStorableVector3(SAVE_PREFIX + GroupID + "-Rotation", new Vector3(0f,0f,0f), new Vector3(0,0,0), new Vector3(0,0,0), false);

			    positionSaveButton = _VAMS.CreateButton("Save Group Position", true);
				_VAMS.setButtonColor(positionSaveButton, unsavedPositionColor );
			    groupComponentsUI.Add((UIDynamic)positionSaveButton);
			    positionSaveButton.button.onClick.AddListener(() => { OnSaveGroupPosition(positionSaveButton); } );
			    
			    // ******* GROUP TRIGGER ***********
			    GroupTrigger = new EventTrigger(_VAMS, SAVE_PREFIX + GroupID + "-GroupTrigger");
			    
			    triggerGroupButton = _VAMS.CreateButton("Edit Group Trigger", true);
			    groupComponentsUI.Add((UIDynamic)triggerGroupButton);
			    triggerGroupButton.button.onClick.AddListener(() => { GroupTrigger.OpenPanelActionStart(); } );
			    
			    // ******* MAIN DIALOG DISABLED ***********
			    GroupHideMainDialog = new JSONStorableBool(SAVE_PREFIX + GroupID + "-HideMainDialog", false);
			    tmpToggle = _VAMS.CreateToggle(GroupHideMainDialog,true);
			    tmpToggle.label = "Hide main dialog at the end";
			    groupComponentsUI.Add((UIDynamic) tmpToggle);
			    
			    // ******* ANSWERS TITLE ***********
			    groupComponentsUI.Add((UIDynamic) _VAMS.createStaticDescriptionText("GroupAnswersTitle_" + GroupID,"\n<color=#000><size=25><b>Answers (displayed with last dialog)</b></size></color>",true,65,TextAnchor.MiddleRight));
			    
			    // ******* ANSWER 01 ***********
			    GroupAnswer01Enabled = new JSONStorableBool(SAVE_PREFIX + GroupID + "-Answer01Enabled", false);
			    tmpToggle = _VAMS.CreateToggle(GroupAnswer01Enabled,true);
			    tmpToggle.label = "Enable answer 01";
			    groupComponentsUI.Add((UIDynamic) tmpToggle);
			    
			    GroupAnswer01Txt = new JSONStorableString(SAVE_PREFIX + GroupID + "-Answer01Text", "_default_");
			    tmpTextField = _VAMS.CreateTextField(GroupAnswer01Txt, true);
			    _VAMS.setupTextField(tmpTextField, 120, false, false);
			    GroupAnswer01TxtInputField = tmpTextField.UItext.gameObject.AddComponent<InputField>();
			    GroupAnswer01TxtInputField.textComponent = tmpTextField.UItext;
			    GroupAnswer01TxtInputField.lineType = InputField.LineType.MultiLineNewline;
			    groupComponentsUI.Add((UIDynamic) tmpTextField);
			    GroupAnswer01Txt.valNoCallback = newDefaultAnswer;
			    GroupAnswer01TxtInputField.text = newDefaultAnswer;				
			    GroupAnswer01TxtInputField.onValueChanged.AddListener( delegate { _VAMS.OnDialogTextChanged(GroupAnswer01Txt,GroupAnswer01TxtInputField); } );
			    
			    GroupAnswer01Trigger = new EventTrigger(_VAMS, SAVE_PREFIX + GroupID + "-Answer01Trigger");
			    
			    triggerAnswer01Button = _VAMS.CreateButton("Edit Answer 01 Trigger", true);
			    groupComponentsUI.Add((UIDynamic)triggerAnswer01Button);
			    triggerAnswer01Button.button.onClick.AddListener(() => { GroupAnswer01Trigger.OpenPanelActionStart(); } );

			    // ******* ANSWER 02 ***********
			    GroupAnswer02Enabled = new JSONStorableBool(SAVE_PREFIX + GroupID + "-Answer02Enabled", false);
			    tmpToggle = _VAMS.CreateToggle(GroupAnswer02Enabled,true);
			    tmpToggle.label = "Enable answer 02";
			    groupComponentsUI.Add((UIDynamic) tmpToggle);
			    
			    GroupAnswer02Txt = new JSONStorableString(SAVE_PREFIX + GroupID + "-Answer02Text", "_default_");
			    tmpTextField = _VAMS.CreateTextField(GroupAnswer02Txt, true);
			    _VAMS.setupTextField(tmpTextField, 120, false, false);
			    GroupAnswer02TxtInputField = tmpTextField.UItext.gameObject.AddComponent<InputField>();
			    GroupAnswer02TxtInputField.textComponent = tmpTextField.UItext;
			    GroupAnswer02TxtInputField.lineType = InputField.LineType.MultiLineNewline;
			    groupComponentsUI.Add((UIDynamic) tmpTextField);
			    GroupAnswer02Txt.valNoCallback = newDefaultAnswer;
			    GroupAnswer02TxtInputField.text = newDefaultAnswer;				
			    GroupAnswer02TxtInputField.onValueChanged.AddListener( delegate { _VAMS.OnDialogTextChanged(GroupAnswer02Txt,GroupAnswer02TxtInputField); } );
			    
			    GroupAnswer02Trigger = new EventTrigger(_VAMS, SAVE_PREFIX + GroupID + "-Answer02Trigger");
			    
			    triggerAnswer02Button = _VAMS.CreateButton("Edit Answer 02 Trigger", true);
			    groupComponentsUI.Add((UIDynamic)triggerAnswer02Button);
			    triggerAnswer02Button.button.onClick.AddListener(() => { GroupAnswer02Trigger.OpenPanelActionStart(); } );
			    
			    // ******* ANSWER 03 ***********
			    GroupAnswer03Enabled = new JSONStorableBool(SAVE_PREFIX + GroupID + "-Answer03Enabled", false);
			    tmpToggle = _VAMS.CreateToggle(GroupAnswer03Enabled,true);
			    tmpToggle.label = "Enable answer 03";
			    groupComponentsUI.Add((UIDynamic) tmpToggle);

			    GroupAnswer03Txt = new JSONStorableString(SAVE_PREFIX + GroupID + "-Answer03Text", "_default_");
			    tmpTextField = _VAMS.CreateTextField(GroupAnswer03Txt, true);
			    _VAMS.setupTextField(tmpTextField, 120, false, false);
			    GroupAnswer03TxtInputField = tmpTextField.UItext.gameObject.AddComponent<InputField>();
			    GroupAnswer03TxtInputField.textComponent = tmpTextField.UItext;
			    GroupAnswer03TxtInputField.lineType = InputField.LineType.MultiLineNewline;
			    groupComponentsUI.Add((UIDynamic) tmpTextField);
			    GroupAnswer03Txt.valNoCallback = newDefaultAnswer;
			    GroupAnswer03TxtInputField.text = newDefaultAnswer;				
			    GroupAnswer03TxtInputField.onValueChanged.AddListener( delegate { _VAMS.OnDialogTextChanged(GroupAnswer03Txt,GroupAnswer03TxtInputField); } );
			    
			    GroupAnswer03Trigger = new EventTrigger(_VAMS, SAVE_PREFIX + GroupID + "-Answer03Trigger");
			    
			    triggerAnswer03Button = _VAMS.CreateButton("Edit Answer 03 Trigger", true);
			    groupComponentsUI.Add((UIDynamic)triggerAnswer03Button);
			    triggerAnswer03Button.button.onClick.AddListener(() => { GroupAnswer03Trigger.OpenPanelActionStart(); } );
			    
			    // ******* GROUP ACTIONS ***********
			    tmpSpacer = _VAMS.CreateSpacer(true);
			    _VAMS.setupSpacer(tmpSpacer, 25f);  
			    groupComponentsUI.Add((UIDynamic) tmpSpacer);
			    
			    GroupStoryActions = new JSONStorableStringChooser(SAVE_PREFIX + GroupID + "-Actions", _VAMS.actionsChoices, "None", "Group Actions");
			    tmpPopup = _VAMS.CreateScrollablePopup(GroupStoryActions,true);
			    tmpPopup.labelWidth = 220f;
			    groupComponentsUI.Add((UIDynamic) tmpPopup);

			    // Registering everything
			    RegisterStorables();
			    
			    // Restoring datas
			    if (dialogsDatas != null)
			    {
					// If you look at VAM's source code, the RestoreFromJSON is super odd
					// and the restore does not work properly for the Vector3
					// so I'm gonna do it manually :|
					// ---- Position
					if( dialogsDatas[GroupPosition.name] != null ) {
						Vector3 savedVectorPos = new Vector3( Single.Parse(dialogsDatas[GroupPosition.name][0]), Single.Parse(dialogsDatas[GroupPosition.name][1]), Single.Parse(dialogsDatas[GroupPosition.name][2]) );
				    	GroupPosition.valNoCallback = savedVectorPos;

						// Changing save position button color if needed
						if( savedVectorPos != default(Vector3) ) {
							_VAMS.setButtonColor(positionSaveButton, savedPositionColor );
						}
					}
					
					// ---- Rotation
					if( dialogsDatas[GroupRotation.name] != null ) {
						Vector3 savedVectorRot = new Vector3( Single.Parse(dialogsDatas[GroupRotation.name][0]), Single.Parse(dialogsDatas[GroupRotation.name][1]), Single.Parse(dialogsDatas[GroupRotation.name][2]) );
						GroupRotation.valNoCallback = savedVectorRot;
					}

					// The other restore working properly
					GroupTrigger.RestoreFromJSON( dialogsDatas, _VAMS.subScenePrefix, _VAMS.mergeRestore, true );
				    GroupAnswer01Enabled.RestoreFromJSON( dialogsDatas );
				    GroupAnswer01Txt.RestoreFromJSON( dialogsDatas, true, true, false );
				    GroupAnswer01Trigger.RestoreFromJSON( dialogsDatas, _VAMS.subScenePrefix, _VAMS.mergeRestore, true );
				    GroupAnswer02Enabled.RestoreFromJSON( dialogsDatas );
				    GroupAnswer02Txt.RestoreFromJSON( dialogsDatas, true, true, false );
				    GroupAnswer02Trigger.RestoreFromJSON( dialogsDatas, _VAMS.subScenePrefix, _VAMS.mergeRestore, true );
				    GroupAnswer03Enabled.RestoreFromJSON( dialogsDatas );
				    GroupAnswer03Txt.RestoreFromJSON( dialogsDatas, true, true, false );
				    GroupAnswer03Trigger.RestoreFromJSON( dialogsDatas, _VAMS.subScenePrefix, _VAMS.mergeRestore, true );
				    GroupStoryActions.RestoreFromJSON( dialogsDatas, true, true, false);
				    GroupHideMainDialog.RestoreFromJSON( dialogsDatas );
			    }
			    
			    // Updating texts inputs
			    GroupAnswer01TxtInputField.text = GroupAnswer01Txt.val;
			    GroupAnswer02TxtInputField.text = GroupAnswer02Txt.val;
			    GroupAnswer03TxtInputField.text = GroupAnswer03Txt.val;
			    
			    ToggleGroupUI(false);
		    }
		    
		    public void UpdateID(int newGroupId)
		    {
			    GroupID = newGroupId;

			    // Updating dialogs
			    int dialCount = 0;
			    foreach( StoryDialog dial in DialogList )
			    {
				    dial.UpdateID(GroupID,dialCount);
				    dialCount++;
			    }
			    
			    // Updating group components (param names for the save)
			    UpdateComponentsIDs();
		    }
		    
		    protected void UpdateComponentsIDs()
		    {
			    // Deregister
			    DeregisterStorables();
			    
			    // Updated
			    GroupPosition.name = SAVE_PREFIX + GroupID + "-Position";
			    GroupRotation.name = SAVE_PREFIX + GroupID + "-Rotation";
			    GroupTrigger.Name = SAVE_PREFIX + GroupID + "-GroupTrigger";
			    GroupAnswer01Enabled.name = SAVE_PREFIX + GroupID + "-Answer01Enabled";
			    GroupAnswer01Txt.name = SAVE_PREFIX + GroupID + "-Answer01Text";
			    GroupAnswer01Trigger.Name = SAVE_PREFIX + GroupID + "-Answer01Trigger";
			    GroupAnswer02Enabled.name = SAVE_PREFIX + GroupID + "-Answer02Enabled";
			    GroupAnswer02Txt.name = SAVE_PREFIX + GroupID + "-Answer02Text";
			    GroupAnswer02Trigger.Name = SAVE_PREFIX + GroupID + "-Answer02Trigger";
			    GroupAnswer03Enabled.name = SAVE_PREFIX + GroupID + "-Answer03Enabled";
			    GroupAnswer03Txt.name = SAVE_PREFIX + GroupID + "-Answer03Text";
			    GroupAnswer03Trigger.Name = SAVE_PREFIX + GroupID + "-Answer03Trigger";
			    GroupStoryActions.name = SAVE_PREFIX + GroupID + "-Actions";
			    GroupHideMainDialog.name = SAVE_PREFIX + GroupID + "-HideMainDialog";
			    
			    // Register
			    RegisterStorables();
		    }
		    
		    public void CreateNewDialog()
		    {
			    DialogList.Add(new StoryDialog(_VAMS, GroupID, DialogList.Count));
		    }

		    public void CreateDialogs(int count)
		    {
			    for (var i = 0; i < count; i++)
			    {
				    DialogList.Add(new StoryDialog(_VAMS, GroupID, i));
			    }
		    }

		    public List<StoryDialog> GetDialogs()
		    {
			    return DialogList;
		    }
		    
		    public StoryDialog GetDialogID(int dialogId)
		    {
			    return DialogList[dialogId];
		    }

			public void DeleteDialogID(int dialogId) {
				StoryDialog targetDial = GetDialogID(dialogId);
				targetDial.Delete();
				DialogList.RemoveAt(dialogId);
				int dialCount = 0;
				foreach( StoryDialog dial in DialogList )
				{
					dial.UpdateID(GroupID,dialCount);
					dialCount++;
				}
			}
		    
		    public int GetDialogCount()
		    {
			    return DialogList.Count;
		    }
		    
		    public bool IsLastDialogID( int dialogId )
		    {
			    return GetDialogCount() == dialogId + 1;
		    }
		    
		    public void ToggleGroupUI( bool enabled )
		    {
			    foreach( UIDynamic uidElem in groupComponentsUI )
			    {
				    _VAMS.ToggleUIDynamic(enabled, uidElem);
			    }
		    }
		    
		    public void ClearDialogsUI()
		    {
			    foreach (StoryDialog dial in DialogList)
			    {
				    dial.ToggleDialogUI(false);
			    }
		    }
		    
		    public void ToggleDialogUI( bool enabled, int dialogId )
		    {
			    DialogList[dialogId].ToggleDialogUI(enabled);
		    }

		    public void DoClickTriggerAnswerID(int answerId)
		    {
			    switch (answerId)
			    {
				    case 1:
					    GroupAnswer01Trigger.Trigger();
					    break;
					case 2:
						GroupAnswer02Trigger.Trigger();
						break;
					case 3:
						GroupAnswer03Trigger.Trigger();
						break;
					default:
						_VAMS.logDebug("VAMStory: you're trying to execute an answer trigger that does not exists.");
						break;
			    }
		    }
		    
		    public void DoClickTriggerDialogID(int dialogId)
		    {
			    DialogList[dialogId].DoClickTrigger();
		    }
		    
		    public void DoShowTriggerDialogID(int dialogId)
		    {
			    DialogList[dialogId].DoShowTrigger();
		    }

			public void MoveGroup() {
				// Don't move if we have the UI following the camera
				if (_VAMS.GlobalOptUIFollowCamera.val == true) return;
				
				// Only moving or rotating if we have a location saved, if not, we just stay were the atom is saved
				if(GroupPosition.val != default(Vector3)) _VAMS.containingAtom.mainController.MoveControl(GroupPosition.val);
				if(GroupRotation.val != default(Vector3)) _VAMS.containingAtom.mainController.RotateControl(GroupRotation.val);
			}

			protected void OnSaveGroupPosition(UIDynamicButton targetBtn ) {
				_VAMS.setButtonColor(targetBtn, savedPositionColor );
				GroupPosition.val = _VAMS.containingAtom.mainController.transform.position;
				GroupRotation.val = _VAMS.containingAtom.mainController.transform.eulerAngles;
				SuperController.singleton.Alert("<b><color=green>VAMStory</color></b>\n\nPosition saved for Group " + GroupID + ".", () => {});
			}
			
			public void Delete() {
				// Before deleting the group I need to clear the dialogs
				foreach( StoryDialog dial in DialogList )
				{
					dial.Delete();
				}

				DialogList.Clear();

				// Clearing the UI
				Destroy(GroupAnswer01TxtInputField);
				Destroy(GroupAnswer02TxtInputField);
				Destroy(GroupAnswer03TxtInputField);
				
				_VAMS.RemoveToggle(GroupAnswer01Enabled);
				_VAMS.RemoveTextField(GroupAnswer01Txt);
				_VAMS.RemoveToggle(GroupAnswer02Enabled);
				_VAMS.RemoveTextField(GroupAnswer02Txt);
				_VAMS.RemoveToggle(GroupAnswer03Enabled);
				_VAMS.RemoveTextField(GroupAnswer03Txt);

				_VAMS.RemoveButton(positionSaveButton);
				_VAMS.RemoveButton(triggerGroupButton);
				_VAMS.RemoveButton(triggerAnswer01Button);
				_VAMS.RemoveButton(triggerAnswer02Button);
				_VAMS.RemoveButton(triggerAnswer03Button);

				// Unregistering vars
				DeregisterStorables();

				// Removing triggers
				GroupTrigger.Remove();
				GroupAnswer01Trigger.Remove();
				GroupAnswer02Trigger.Remove();
				GroupAnswer03Trigger.Remove();

				// Clearing data from the loaded data of the scene (to prevent re-restore after deleting)
				// This only happens when you load a scene and have data loaded from the json
				if (dialogsDatas != null)
				{
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Position");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Rotation");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-GroupTrigger");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Answer01Enabled");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Answer01Text");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Answer01Trigger");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Answer02Enabled");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Answer02Text");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Answer02Trigger");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Answer03Enabled");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Answer03Text");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Answer03Trigger");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-Actions");
					dialogsDatas.Remove(SAVE_PREFIX + GroupID + "-HideMainDialog");
				}
			}
			
			protected void DeregisterStorables()
			{
				_VAMS.DeregisterVector3(GroupPosition);
				_VAMS.DeregisterVector3(GroupRotation);
			
				_VAMS.DeregisterBool(GroupAnswer01Enabled);
				_VAMS.DeregisterString(GroupAnswer01Txt);
				_VAMS.DeregisterBool(GroupAnswer02Enabled);
				_VAMS.DeregisterString(GroupAnswer02Txt);
				_VAMS.DeregisterBool(GroupAnswer03Enabled);
				_VAMS.DeregisterString(GroupAnswer03Txt);
				_VAMS.DeregisterStringChooser(GroupStoryActions);
				_VAMS.DeregisterBool(GroupHideMainDialog);
			}
			
			protected void RegisterStorables()
			{
				_VAMS.RegisterVector3(GroupPosition);
				_VAMS.RegisterVector3(GroupRotation);
			
				_VAMS.RegisterBool(GroupAnswer01Enabled);
				_VAMS.RegisterString(GroupAnswer01Txt);
				_VAMS.RegisterBool(GroupAnswer02Enabled);
				_VAMS.RegisterString(GroupAnswer02Txt);
				_VAMS.RegisterBool(GroupAnswer03Enabled);
				_VAMS.RegisterString(GroupAnswer03Txt);
				_VAMS.RegisterStringChooser(GroupStoryActions);
				_VAMS.RegisterBool(GroupHideMainDialog);
			}
	    }
    }
}